/* sysvipc.h -- Interface to sysvipc.c
   Copyright (c) 1998 by Holger Veit

This file is part of emx.

emx is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

emx is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with emx; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

As special exception, emx.dll can be distributed without source code
unless it has been changed.  If you modify emx.dll, this exception
no longer applies and you must remove this paragraph from all source
files for emx.dll.  */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/msg.h>

/*
 *     H A C K S
 */

/* to be replaced by real UID/GID when it exists, currently running as root */
#define ipc_uid() (0)
#define ipc_gid() (0)
#define ipc_pid() __getpid()
#define S_IRUGO (S_IRUSR|S_IRGRP|S_IROTH)
#define S_IWUGO (S_IWUSR|S_IWGRP|S_IWOTH)

/*
 *      S H M   section
 */
struct _shmitem
{
  struct shmid_ds	ds;	/* this is exported to the user */
  short                 flags;  /* special flags */
  void*			ptr;	/* memory pointer */
};

struct _shmcommon {
  unsigned short        seqno;
  unsigned short        flags;
  short                 maxid;
  short                 usecnt;
  struct _shmitem       shm[1];
};

#define SHMMIN		4096			/* page size (4K) */
#define SHMALL		64*1024*SHMMIN/4	/* max number of SHM pages (64MB) */
#define SHMMAX		4*1024*1024		/* max shm seg size (4MB) */
#define SHMMNI		4096			/* max shm slots */
#define SHM_UNUSED      0xdeaddead		/* don't use this key! */
#define SHM_INUSE       0x00000001		/* _shmitem.flags */
#define SHM_DELSEG	0x00000002		/* _shmitem.flags */

struct _shmargs;
int sysv_shmat(struct _shmargs* args, int *errnop);
int sysv_shmctl(struct _shmargs* args, int *errnop);
int sysv_shmdt(struct _shmargs* args, int *errnop);
int sysv_shmget(struct _shmargs* args, int *errnop);
int shm_attach();
void shm_detach();
int shm_newid();
void shm_delseg(struct _shmitem* shm);

/*
 *      S E M   section
 */
struct _semitem
{
  struct semid_ds	ds;	/* this is exported to the user */
};

struct _semcommon {
  unsigned short       seqno;
  unsigned short       flags;
  short                usecnt;
  struct _semitem sem[1];
};

#define SEMMNI		512	/* max number of semaphore sets */
#define SEMMSL		512	/* max number of semaphores per set */
#define SEMMNS		SEMMNI*SEMMSL /* total max no of semaphores */

struct _semargs;
int sysv_semctl(struct _semargs* args, int *errnop);
int sysv_semget(struct _semargs* args, int *errnop);
int sysv_semop(struct _semargs* args, int *errnop);
int sysv_semconfig(struct _semargs* args, int *errnop);
int sem_attach();
void sem_detach();

/*
 *     M S G   section
 */
struct _msgitem
{
  struct msqid_ds	ds;	/* this is exported to the user */
};

struct _msgcommon {
  unsigned short       seqno;
  unsigned short       flags;
  short                usecnt;
  struct _msgitem      msg[1];
};

#define MSGMNI		128	/* number of available msg queues */

struct _msgargs;
int sysv_msgctl(struct _msgargs* args, int *errnop);
int sysv_msgget(struct _msgargs* args, int *errnop);
int sysv_msgrcv(struct _msgargs* args, int *errnop);
int sysv_msgsnd(struct _msgargs* args, int *errnop);
int msg_attach();
void msg_detach();

/*
 *  shared functions
 */
void init_sysvipc(void);
void term_sysvipc(void);
int sysv_attachglobal(const char*,const char*,int,PPVOID,PHMTX);
void sysv_detachglobal(PVOID,HMTX);
int ipc_open(key_t,int,int,int,struct ipc_perm*,int*);
int ipc_access(struct ipc_perm* perm, int flags);